@@ -1,7 +1,7 @@
 """
 Type annotations for memorydb service type definitions.
 
-[Open documentation](https://vemel.github.io/boto3_stubs_docs/mypy_boto3_memorydb/type_defs/)
+[Open documentation](https://youtype.github.io/boto3_stubs_docs/mypy_boto3_memorydb/type_defs/)
 
 Usage::
 
@@ -140,61 +140,81 @@ __all__ = (
 ACLPendingChangesTypeDef = TypedDict(
     "ACLPendingChangesTypeDef",
     {
-        "UserNamesToRemove": NotRequired[List[str]],
-        "UserNamesToAdd": NotRequired[List[str]],
+        "UserNamesToRemove": List[str],
+        "UserNamesToAdd": List[str],
     },
+    total=False,
 )
 
 ACLTypeDef = TypedDict(
     "ACLTypeDef",
     {
-        "Name": NotRequired[str],
-        "Status": NotRequired[str],
-        "UserNames": NotRequired[List[str]],
-        "MinimumEngineVersion": NotRequired[str],
-        "PendingChanges": NotRequired["ACLPendingChangesTypeDef"],
-        "Clusters": NotRequired[List[str]],
-        "ARN": NotRequired[str],
+        "Name": str,
+        "Status": str,
+        "UserNames": List[str],
+        "MinimumEngineVersion": str,
+        "PendingChanges": "ACLPendingChangesTypeDef",
+        "Clusters": List[str],
+        "ARN": str,
     },
+    total=False,
 )
 
 ACLsUpdateStatusTypeDef = TypedDict(
     "ACLsUpdateStatusTypeDef",
     {
-        "ACLToApply": NotRequired[str],
+        "ACLToApply": str,
     },
+    total=False,
 )
 
 AuthenticationModeTypeDef = TypedDict(
     "AuthenticationModeTypeDef",
     {
-        "Type": NotRequired[Literal["password"]],
-        "Passwords": NotRequired[Sequence[str]],
+        "Type": Literal["password"],
+        "Passwords": Sequence[str],
     },
+    total=False,
 )
 
 AuthenticationTypeDef = TypedDict(
     "AuthenticationTypeDef",
     {
-        "Type": NotRequired[AuthenticationTypeType],
-        "PasswordCount": NotRequired[int],
+        "Type": AuthenticationTypeType,
+        "PasswordCount": int,
     },
+    total=False,
 )
 
 AvailabilityZoneTypeDef = TypedDict(
     "AvailabilityZoneTypeDef",
     {
-        "Name": NotRequired[str],
+        "Name": str,
     },
+    total=False,
 )
 
-BatchUpdateClusterRequestRequestTypeDef = TypedDict(
-    "BatchUpdateClusterRequestRequestTypeDef",
+_RequiredBatchUpdateClusterRequestRequestTypeDef = TypedDict(
+    "_RequiredBatchUpdateClusterRequestRequestTypeDef",
     {
         "ClusterNames": Sequence[str],
-        "ServiceUpdate": NotRequired["ServiceUpdateRequestTypeDef"],
     },
 )
+_OptionalBatchUpdateClusterRequestRequestTypeDef = TypedDict(
+    "_OptionalBatchUpdateClusterRequestRequestTypeDef",
+    {
+        "ServiceUpdate": "ServiceUpdateRequestTypeDef",
+    },
+    total=False,
+)
+
+
+class BatchUpdateClusterRequestRequestTypeDef(
+    _RequiredBatchUpdateClusterRequestRequestTypeDef,
+    _OptionalBatchUpdateClusterRequestRequestTypeDef,
+):
+    pass
+
 
 BatchUpdateClusterResponseTypeDef = TypedDict(
     "BatchUpdateClusterResponseTypeDef",
@@ -208,73 +228,89 @@ BatchUpdateClusterResponseTypeDef = TypedDict(
 ClusterConfigurationTypeDef = TypedDict(
     "ClusterConfigurationTypeDef",
     {
-        "Name": NotRequired[str],
-        "Description": NotRequired[str],
-        "NodeType": NotRequired[str],
-        "EngineVersion": NotRequired[str],
-        "MaintenanceWindow": NotRequired[str],
-        "TopicArn": NotRequired[str],
-        "Port": NotRequired[int],
-        "ParameterGroupName": NotRequired[str],
-        "SubnetGroupName": NotRequired[str],
-        "VpcId": NotRequired[str],
-        "SnapshotRetentionLimit": NotRequired[int],
-        "SnapshotWindow": NotRequired[str],
-        "NumShards": NotRequired[int],
-        "Shards": NotRequired[List["ShardDetailTypeDef"]],
+        "Name": str,
+        "Description": str,
+        "NodeType": str,
+        "EngineVersion": str,
+        "MaintenanceWindow": str,
+        "TopicArn": str,
+        "Port": int,
+        "ParameterGroupName": str,
+        "SubnetGroupName": str,
+        "VpcId": str,
+        "SnapshotRetentionLimit": int,
+        "SnapshotWindow": str,
+        "NumShards": int,
+        "Shards": List["ShardDetailTypeDef"],
     },
+    total=False,
 )
 
 ClusterPendingUpdatesTypeDef = TypedDict(
     "ClusterPendingUpdatesTypeDef",
     {
-        "Resharding": NotRequired["ReshardingStatusTypeDef"],
-        "ACLs": NotRequired["ACLsUpdateStatusTypeDef"],
-        "ServiceUpdates": NotRequired[List["PendingModifiedServiceUpdateTypeDef"]],
+        "Resharding": "ReshardingStatusTypeDef",
+        "ACLs": "ACLsUpdateStatusTypeDef",
+        "ServiceUpdates": List["PendingModifiedServiceUpdateTypeDef"],
     },
+    total=False,
 )
 
 ClusterTypeDef = TypedDict(
     "ClusterTypeDef",
     {
-        "Name": NotRequired[str],
-        "Description": NotRequired[str],
-        "Status": NotRequired[str],
-        "PendingUpdates": NotRequired["ClusterPendingUpdatesTypeDef"],
-        "NumberOfShards": NotRequired[int],
-        "Shards": NotRequired[List["ShardTypeDef"]],
-        "AvailabilityMode": NotRequired[AZStatusType],
-        "ClusterEndpoint": NotRequired["EndpointTypeDef"],
-        "NodeType": NotRequired[str],
-        "EngineVersion": NotRequired[str],
-        "EnginePatchVersion": NotRequired[str],
-        "ParameterGroupName": NotRequired[str],
-        "ParameterGroupStatus": NotRequired[str],
-        "SecurityGroups": NotRequired[List["SecurityGroupMembershipTypeDef"]],
-        "SubnetGroupName": NotRequired[str],
-        "TLSEnabled": NotRequired[bool],
-        "KmsKeyId": NotRequired[str],
-        "ARN": NotRequired[str],
-        "SnsTopicArn": NotRequired[str],
-        "SnsTopicStatus": NotRequired[str],
-        "SnapshotRetentionLimit": NotRequired[int],
-        "MaintenanceWindow": NotRequired[str],
-        "SnapshotWindow": NotRequired[str],
-        "ACLName": NotRequired[str],
-        "AutoMinorVersionUpgrade": NotRequired[bool],
+        "Name": str,
+        "Description": str,
+        "Status": str,
+        "PendingUpdates": "ClusterPendingUpdatesTypeDef",
+        "NumberOfShards": int,
+        "Shards": List["ShardTypeDef"],
+        "AvailabilityMode": AZStatusType,
+        "ClusterEndpoint": "EndpointTypeDef",
+        "NodeType": str,
+        "EngineVersion": str,
+        "EnginePatchVersion": str,
+        "ParameterGroupName": str,
+        "ParameterGroupStatus": str,
+        "SecurityGroups": List["SecurityGroupMembershipTypeDef"],
+        "SubnetGroupName": str,
+        "TLSEnabled": bool,
+        "KmsKeyId": str,
+        "ARN": str,
+        "SnsTopicArn": str,
+        "SnsTopicStatus": str,
+        "SnapshotRetentionLimit": int,
+        "MaintenanceWindow": str,
+        "SnapshotWindow": str,
+        "ACLName": str,
+        "AutoMinorVersionUpgrade": bool,
     },
+    total=False,
 )
 
-CopySnapshotRequestRequestTypeDef = TypedDict(
-    "CopySnapshotRequestRequestTypeDef",
+_RequiredCopySnapshotRequestRequestTypeDef = TypedDict(
+    "_RequiredCopySnapshotRequestRequestTypeDef",
     {
         "SourceSnapshotName": str,
         "TargetSnapshotName": str,
-        "TargetBucket": NotRequired[str],
-        "KmsKeyId": NotRequired[str],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCopySnapshotRequestRequestTypeDef = TypedDict(
+    "_OptionalCopySnapshotRequestRequestTypeDef",
+    {
+        "TargetBucket": str,
+        "KmsKeyId": str,
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CopySnapshotRequestRequestTypeDef(
+    _RequiredCopySnapshotRequestRequestTypeDef, _OptionalCopySnapshotRequestRequestTypeDef
+):
+    pass
+
 
 CopySnapshotResponseTypeDef = TypedDict(
     "CopySnapshotResponseTypeDef",
@@ -284,14 +320,27 @@ CopySnapshotResponseTypeDef = TypedDict(
     },
 )
 
-CreateACLRequestRequestTypeDef = TypedDict(
-    "CreateACLRequestRequestTypeDef",
+_RequiredCreateACLRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateACLRequestRequestTypeDef",
     {
         "ACLName": str,
-        "UserNames": NotRequired[Sequence[str]],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCreateACLRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateACLRequestRequestTypeDef",
+    {
+        "UserNames": Sequence[str],
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CreateACLRequestRequestTypeDef(
+    _RequiredCreateACLRequestRequestTypeDef, _OptionalCreateACLRequestRequestTypeDef
+):
+    pass
+
 
 CreateACLResponseTypeDef = TypedDict(
     "CreateACLResponseTypeDef",
@@ -301,32 +350,45 @@ CreateACLResponseTypeDef = TypedDict(
     },
 )
 
-CreateClusterRequestRequestTypeDef = TypedDict(
-    "CreateClusterRequestRequestTypeDef",
+_RequiredCreateClusterRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
         "NodeType": str,
         "ACLName": str,
-        "ParameterGroupName": NotRequired[str],
-        "Description": NotRequired[str],
-        "NumShards": NotRequired[int],
-        "NumReplicasPerShard": NotRequired[int],
-        "SubnetGroupName": NotRequired[str],
-        "SecurityGroupIds": NotRequired[Sequence[str]],
-        "MaintenanceWindow": NotRequired[str],
-        "Port": NotRequired[int],
-        "SnsTopicArn": NotRequired[str],
-        "TLSEnabled": NotRequired[bool],
-        "KmsKeyId": NotRequired[str],
-        "SnapshotArns": NotRequired[Sequence[str]],
-        "SnapshotName": NotRequired[str],
-        "SnapshotRetentionLimit": NotRequired[int],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
-        "SnapshotWindow": NotRequired[str],
-        "EngineVersion": NotRequired[str],
-        "AutoMinorVersionUpgrade": NotRequired[bool],
     },
 )
+_OptionalCreateClusterRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateClusterRequestRequestTypeDef",
+    {
+        "ParameterGroupName": str,
+        "Description": str,
+        "NumShards": int,
+        "NumReplicasPerShard": int,
+        "SubnetGroupName": str,
+        "SecurityGroupIds": Sequence[str],
+        "MaintenanceWindow": str,
+        "Port": int,
+        "SnsTopicArn": str,
+        "TLSEnabled": bool,
+        "KmsKeyId": str,
+        "SnapshotArns": Sequence[str],
+        "SnapshotName": str,
+        "SnapshotRetentionLimit": int,
+        "Tags": Sequence["TagTypeDef"],
+        "SnapshotWindow": str,
+        "EngineVersion": str,
+        "AutoMinorVersionUpgrade": bool,
+    },
+    total=False,
+)
+
+
+class CreateClusterRequestRequestTypeDef(
+    _RequiredCreateClusterRequestRequestTypeDef, _OptionalCreateClusterRequestRequestTypeDef
+):
+    pass
+
 
 CreateClusterResponseTypeDef = TypedDict(
     "CreateClusterResponseTypeDef",
@@ -336,15 +398,29 @@ CreateClusterResponseTypeDef = TypedDict(
     },
 )
 
-CreateParameterGroupRequestRequestTypeDef = TypedDict(
-    "CreateParameterGroupRequestRequestTypeDef",
+_RequiredCreateParameterGroupRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateParameterGroupRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
         "Family": str,
-        "Description": NotRequired[str],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCreateParameterGroupRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateParameterGroupRequestRequestTypeDef",
+    {
+        "Description": str,
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CreateParameterGroupRequestRequestTypeDef(
+    _RequiredCreateParameterGroupRequestRequestTypeDef,
+    _OptionalCreateParameterGroupRequestRequestTypeDef,
+):
+    pass
+
 
 CreateParameterGroupResponseTypeDef = TypedDict(
     "CreateParameterGroupResponseTypeDef",
@@ -354,15 +430,28 @@ CreateParameterGroupResponseTypeDef = TypedDict(
     },
 )
 
-CreateSnapshotRequestRequestTypeDef = TypedDict(
-    "CreateSnapshotRequestRequestTypeDef",
+_RequiredCreateSnapshotRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateSnapshotRequestRequestTypeDef",
     {
         "ClusterName": str,
         "SnapshotName": str,
-        "KmsKeyId": NotRequired[str],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCreateSnapshotRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateSnapshotRequestRequestTypeDef",
+    {
+        "KmsKeyId": str,
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CreateSnapshotRequestRequestTypeDef(
+    _RequiredCreateSnapshotRequestRequestTypeDef, _OptionalCreateSnapshotRequestRequestTypeDef
+):
+    pass
+
 
 CreateSnapshotResponseTypeDef = TypedDict(
     "CreateSnapshotResponseTypeDef",
@@ -372,15 +461,28 @@ CreateSnapshotResponseTypeDef = TypedDict(
     },
 )
 
-CreateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "CreateSubnetGroupRequestRequestTypeDef",
+_RequiredCreateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateSubnetGroupRequestRequestTypeDef",
     {
         "SubnetGroupName": str,
         "SubnetIds": Sequence[str],
-        "Description": NotRequired[str],
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCreateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateSubnetGroupRequestRequestTypeDef",
+    {
+        "Description": str,
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CreateSubnetGroupRequestRequestTypeDef(
+    _RequiredCreateSubnetGroupRequestRequestTypeDef, _OptionalCreateSubnetGroupRequestRequestTypeDef
+):
+    pass
+
 
 CreateSubnetGroupResponseTypeDef = TypedDict(
     "CreateSubnetGroupResponseTypeDef",
@@ -390,15 +492,28 @@ CreateSubnetGroupResponseTypeDef = TypedDict(
     },
 )
 
-CreateUserRequestRequestTypeDef = TypedDict(
-    "CreateUserRequestRequestTypeDef",
+_RequiredCreateUserRequestRequestTypeDef = TypedDict(
+    "_RequiredCreateUserRequestRequestTypeDef",
     {
         "UserName": str,
         "AuthenticationMode": "AuthenticationModeTypeDef",
         "AccessString": str,
-        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
+_OptionalCreateUserRequestRequestTypeDef = TypedDict(
+    "_OptionalCreateUserRequestRequestTypeDef",
+    {
+        "Tags": Sequence["TagTypeDef"],
+    },
+    total=False,
+)
+
+
+class CreateUserRequestRequestTypeDef(
+    _RequiredCreateUserRequestRequestTypeDef, _OptionalCreateUserRequestRequestTypeDef
+):
+    pass
+
 
 CreateUserResponseTypeDef = TypedDict(
     "CreateUserResponseTypeDef",
@@ -423,13 +538,26 @@ DeleteACLResponseTypeDef = TypedDict(
     },
 )
 
-DeleteClusterRequestRequestTypeDef = TypedDict(
-    "DeleteClusterRequestRequestTypeDef",
+_RequiredDeleteClusterRequestRequestTypeDef = TypedDict(
+    "_RequiredDeleteClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
-        "FinalSnapshotName": NotRequired[str],
     },
 )
+_OptionalDeleteClusterRequestRequestTypeDef = TypedDict(
+    "_OptionalDeleteClusterRequestRequestTypeDef",
+    {
+        "FinalSnapshotName": str,
+    },
+    total=False,
+)
+
+
+class DeleteClusterRequestRequestTypeDef(
+    _RequiredDeleteClusterRequestRequestTypeDef, _OptionalDeleteClusterRequestRequestTypeDef
+):
+    pass
+
 
 DeleteClusterResponseTypeDef = TypedDict(
     "DeleteClusterResponseTypeDef",
@@ -502,10 +630,11 @@ DeleteUserResponseTypeDef = TypedDict(
 DescribeACLsRequestRequestTypeDef = TypedDict(
     "DescribeACLsRequestRequestTypeDef",
     {
-        "ACLName": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "ACLName": str,
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeACLsResponseTypeDef = TypedDict(
@@ -520,11 +649,12 @@ DescribeACLsResponseTypeDef = TypedDict(
 DescribeClustersRequestRequestTypeDef = TypedDict(
     "DescribeClustersRequestRequestTypeDef",
     {
-        "ClusterName": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
-        "ShowShardDetails": NotRequired[bool],
+        "ClusterName": str,
+        "MaxResults": int,
+        "NextToken": str,
+        "ShowShardDetails": bool,
     },
+    total=False,
 )
 
 DescribeClustersResponseTypeDef = TypedDict(
@@ -539,12 +669,13 @@ DescribeClustersResponseTypeDef = TypedDict(
 DescribeEngineVersionsRequestRequestTypeDef = TypedDict(
     "DescribeEngineVersionsRequestRequestTypeDef",
     {
-        "EngineVersion": NotRequired[str],
-        "ParameterGroupFamily": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
-        "DefaultOnly": NotRequired[bool],
+        "EngineVersion": str,
+        "ParameterGroupFamily": str,
+        "MaxResults": int,
+        "NextToken": str,
+        "DefaultOnly": bool,
     },
+    total=False,
 )
 
 DescribeEngineVersionsResponseTypeDef = TypedDict(
@@ -559,14 +690,15 @@ DescribeEngineVersionsResponseTypeDef = TypedDict(
 DescribeEventsRequestRequestTypeDef = TypedDict(
     "DescribeEventsRequestRequestTypeDef",
     {
-        "SourceName": NotRequired[str],
-        "SourceType": NotRequired[SourceTypeType],
-        "StartTime": NotRequired[Union[datetime, str]],
-        "EndTime": NotRequired[Union[datetime, str]],
-        "Duration": NotRequired[int],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "SourceName": str,
+        "SourceType": SourceTypeType,
+        "StartTime": Union[datetime, str],
+        "EndTime": Union[datetime, str],
+        "Duration": int,
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeEventsResponseTypeDef = TypedDict(
@@ -581,10 +713,11 @@ DescribeEventsResponseTypeDef = TypedDict(
 DescribeParameterGroupsRequestRequestTypeDef = TypedDict(
     "DescribeParameterGroupsRequestRequestTypeDef",
     {
-        "ParameterGroupName": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "ParameterGroupName": str,
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeParameterGroupsResponseTypeDef = TypedDict(
@@ -596,14 +729,28 @@ DescribeParameterGroupsResponseTypeDef = TypedDict(
     },
 )
 
-DescribeParametersRequestRequestTypeDef = TypedDict(
-    "DescribeParametersRequestRequestTypeDef",
+_RequiredDescribeParametersRequestRequestTypeDef = TypedDict(
+    "_RequiredDescribeParametersRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
     },
 )
+_OptionalDescribeParametersRequestRequestTypeDef = TypedDict(
+    "_OptionalDescribeParametersRequestRequestTypeDef",
+    {
+        "MaxResults": int,
+        "NextToken": str,
+    },
+    total=False,
+)
+
+
+class DescribeParametersRequestRequestTypeDef(
+    _RequiredDescribeParametersRequestRequestTypeDef,
+    _OptionalDescribeParametersRequestRequestTypeDef,
+):
+    pass
+
 
 DescribeParametersResponseTypeDef = TypedDict(
     "DescribeParametersResponseTypeDef",
@@ -617,12 +764,13 @@ DescribeParametersResponseTypeDef = TypedDict(
 DescribeServiceUpdatesRequestRequestTypeDef = TypedDict(
     "DescribeServiceUpdatesRequestRequestTypeDef",
     {
-        "ServiceUpdateName": NotRequired[str],
-        "ClusterNames": NotRequired[Sequence[str]],
-        "Status": NotRequired[Sequence[ServiceUpdateStatusType]],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "ServiceUpdateName": str,
+        "ClusterNames": Sequence[str],
+        "Status": Sequence[ServiceUpdateStatusType],
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeServiceUpdatesResponseTypeDef = TypedDict(
@@ -637,13 +785,14 @@ DescribeServiceUpdatesResponseTypeDef = TypedDict(
 DescribeSnapshotsRequestRequestTypeDef = TypedDict(
     "DescribeSnapshotsRequestRequestTypeDef",
     {
-        "ClusterName": NotRequired[str],
-        "SnapshotName": NotRequired[str],
-        "Source": NotRequired[str],
-        "NextToken": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "ShowDetail": NotRequired[bool],
+        "ClusterName": str,
+        "SnapshotName": str,
+        "Source": str,
+        "NextToken": str,
+        "MaxResults": int,
+        "ShowDetail": bool,
     },
+    total=False,
 )
 
 DescribeSnapshotsResponseTypeDef = TypedDict(
@@ -658,10 +807,11 @@ DescribeSnapshotsResponseTypeDef = TypedDict(
 DescribeSubnetGroupsRequestRequestTypeDef = TypedDict(
     "DescribeSubnetGroupsRequestRequestTypeDef",
     {
-        "SubnetGroupName": NotRequired[str],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "SubnetGroupName": str,
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeSubnetGroupsResponseTypeDef = TypedDict(
@@ -676,11 +826,12 @@ DescribeSubnetGroupsResponseTypeDef = TypedDict(
 DescribeUsersRequestRequestTypeDef = TypedDict(
     "DescribeUsersRequestRequestTypeDef",
     {
-        "UserName": NotRequired[str],
-        "Filters": NotRequired[Sequence["FilterTypeDef"]],
-        "MaxResults": NotRequired[int],
-        "NextToken": NotRequired[str],
+        "UserName": str,
+        "Filters": Sequence["FilterTypeDef"],
+        "MaxResults": int,
+        "NextToken": str,
     },
+    total=False,
 )
 
 DescribeUsersResponseTypeDef = TypedDict(
@@ -695,28 +846,31 @@ DescribeUsersResponseTypeDef = TypedDict(
 EndpointTypeDef = TypedDict(
     "EndpointTypeDef",
     {
-        "Address": NotRequired[str],
-        "Port": NotRequired[int],
+        "Address": str,
+        "Port": int,
     },
+    total=False,
 )
 
 EngineVersionInfoTypeDef = TypedDict(
     "EngineVersionInfoTypeDef",
     {
-        "EngineVersion": NotRequired[str],
-        "EnginePatchVersion": NotRequired[str],
-        "ParameterGroupFamily": NotRequired[str],
+        "EngineVersion": str,
+        "EnginePatchVersion": str,
+        "ParameterGroupFamily": str,
     },
+    total=False,
 )
 
 EventTypeDef = TypedDict(
     "EventTypeDef",
     {
-        "SourceName": NotRequired[str],
-        "SourceType": NotRequired[SourceTypeType],
-        "Message": NotRequired[str],
-        "Date": NotRequired[datetime],
+        "SourceName": str,
+        "SourceType": SourceTypeType,
+        "Message": str,
+        "Date": datetime,
     },
+    total=False,
 )
 
 FailoverShardRequestRequestTypeDef = TypedDict(
@@ -777,67 +931,87 @@ ListTagsResponseTypeDef = TypedDict(
 NodeTypeDef = TypedDict(
     "NodeTypeDef",
     {
-        "Name": NotRequired[str],
-        "Status": NotRequired[str],
-        "AvailabilityZone": NotRequired[str],
-        "CreateTime": NotRequired[datetime],
-        "Endpoint": NotRequired["EndpointTypeDef"],
+        "Name": str,
+        "Status": str,
+        "AvailabilityZone": str,
+        "CreateTime": datetime,
+        "Endpoint": "EndpointTypeDef",
     },
+    total=False,
 )
 
 ParameterGroupTypeDef = TypedDict(
     "ParameterGroupTypeDef",
     {
-        "Name": NotRequired[str],
-        "Family": NotRequired[str],
-        "Description": NotRequired[str],
-        "ARN": NotRequired[str],
+        "Name": str,
+        "Family": str,
+        "Description": str,
+        "ARN": str,
     },
+    total=False,
 )
 
 ParameterNameValueTypeDef = TypedDict(
     "ParameterNameValueTypeDef",
     {
-        "ParameterName": NotRequired[str],
-        "ParameterValue": NotRequired[str],
+        "ParameterName": str,
+        "ParameterValue": str,
     },
+    total=False,
 )
 
 ParameterTypeDef = TypedDict(
     "ParameterTypeDef",
     {
-        "Name": NotRequired[str],
-        "Value": NotRequired[str],
-        "Description": NotRequired[str],
-        "DataType": NotRequired[str],
-        "AllowedValues": NotRequired[str],
-        "MinimumEngineVersion": NotRequired[str],
+        "Name": str,
+        "Value": str,
+        "Description": str,
+        "DataType": str,
+        "AllowedValues": str,
+        "MinimumEngineVersion": str,
     },
+    total=False,
 )
 
 PendingModifiedServiceUpdateTypeDef = TypedDict(
     "PendingModifiedServiceUpdateTypeDef",
     {
-        "ServiceUpdateName": NotRequired[str],
-        "Status": NotRequired[ServiceUpdateStatusType],
+        "ServiceUpdateName": str,
+        "Status": ServiceUpdateStatusType,
     },
+    total=False,
 )
 
 ReplicaConfigurationRequestTypeDef = TypedDict(
     "ReplicaConfigurationRequestTypeDef",
     {
-        "ReplicaCount": NotRequired[int],
+        "ReplicaCount": int,
     },
+    total=False,
 )
 
-ResetParameterGroupRequestRequestTypeDef = TypedDict(
-    "ResetParameterGroupRequestRequestTypeDef",
+_RequiredResetParameterGroupRequestRequestTypeDef = TypedDict(
+    "_RequiredResetParameterGroupRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
-        "AllParameters": NotRequired[bool],
-        "ParameterNames": NotRequired[Sequence[str]],
     },
 )
+_OptionalResetParameterGroupRequestRequestTypeDef = TypedDict(
+    "_OptionalResetParameterGroupRequestRequestTypeDef",
+    {
+        "AllParameters": bool,
+        "ParameterNames": Sequence[str],
+    },
+    total=False,
+)
+
+
+class ResetParameterGroupRequestRequestTypeDef(
+    _RequiredResetParameterGroupRequestRequestTypeDef,
+    _OptionalResetParameterGroupRequestRequestTypeDef,
+):
+    pass
+
 
 ResetParameterGroupResponseTypeDef = TypedDict(
     "ResetParameterGroupResponseTypeDef",
@@ -850,8 +1024,9 @@ ResetParameterGroupResponseTypeDef = TypedDict(
 ReshardingStatusTypeDef = TypedDict(
     "ReshardingStatusTypeDef",
     {
-        "SlotMigration": NotRequired["SlotMigrationTypeDef"],
+        "SlotMigration": "SlotMigrationTypeDef",
     },
+    total=False,
 )
 
 ResponseMetadataTypeDef = TypedDict(
@@ -868,104 +1043,115 @@ ResponseMetadataTypeDef = TypedDict(
 SecurityGroupMembershipTypeDef = TypedDict(
     "SecurityGroupMembershipTypeDef",
     {
-        "SecurityGroupId": NotRequired[str],
-        "Status": NotRequired[str],
+        "SecurityGroupId": str,
+        "Status": str,
     },
+    total=False,
 )
 
 ServiceUpdateRequestTypeDef = TypedDict(
     "ServiceUpdateRequestTypeDef",
     {
-        "ServiceUpdateNameToApply": NotRequired[str],
+        "ServiceUpdateNameToApply": str,
     },
+    total=False,
 )
 
 ServiceUpdateTypeDef = TypedDict(
     "ServiceUpdateTypeDef",
     {
-        "ClusterName": NotRequired[str],
-        "ServiceUpdateName": NotRequired[str],
-        "ReleaseDate": NotRequired[datetime],
-        "Description": NotRequired[str],
-        "Status": NotRequired[ServiceUpdateStatusType],
-        "Type": NotRequired[Literal["security-update"]],
-        "NodesUpdated": NotRequired[str],
-        "AutoUpdateStartDate": NotRequired[datetime],
+        "ClusterName": str,
+        "ServiceUpdateName": str,
+        "ReleaseDate": datetime,
+        "Description": str,
+        "Status": ServiceUpdateStatusType,
+        "Type": Literal["security-update"],
+        "NodesUpdated": str,
+        "AutoUpdateStartDate": datetime,
     },
+    total=False,
 )
 
 ShardConfigurationRequestTypeDef = TypedDict(
     "ShardConfigurationRequestTypeDef",
     {
-        "ShardCount": NotRequired[int],
+        "ShardCount": int,
     },
+    total=False,
 )
 
 ShardConfigurationTypeDef = TypedDict(
     "ShardConfigurationTypeDef",
     {
-        "Slots": NotRequired[str],
-        "ReplicaCount": NotRequired[int],
+        "Slots": str,
+        "ReplicaCount": int,
     },
+    total=False,
 )
 
 ShardDetailTypeDef = TypedDict(
     "ShardDetailTypeDef",
     {
-        "Name": NotRequired[str],
-        "Configuration": NotRequired["ShardConfigurationTypeDef"],
-        "Size": NotRequired[str],
-        "SnapshotCreationTime": NotRequired[datetime],
+        "Name": str,
+        "Configuration": "ShardConfigurationTypeDef",
+        "Size": str,
+        "SnapshotCreationTime": datetime,
     },
+    total=False,
 )
 
 ShardTypeDef = TypedDict(
     "ShardTypeDef",
     {
-        "Name": NotRequired[str],
-        "Status": NotRequired[str],
-        "Slots": NotRequired[str],
-        "Nodes": NotRequired[List["NodeTypeDef"]],
-        "NumberOfNodes": NotRequired[int],
+        "Name": str,
+        "Status": str,
+        "Slots": str,
+        "Nodes": List["NodeTypeDef"],
+        "NumberOfNodes": int,
     },
+    total=False,
 )
 
 SlotMigrationTypeDef = TypedDict(
     "SlotMigrationTypeDef",
     {
-        "ProgressPercentage": NotRequired[float],
+        "ProgressPercentage": float,
     },
+    total=False,
 )
 
 SnapshotTypeDef = TypedDict(
     "SnapshotTypeDef",
     {
-        "Name": NotRequired[str],
-        "Status": NotRequired[str],
-        "Source": NotRequired[str],
-        "KmsKeyId": NotRequired[str],
-        "ARN": NotRequired[str],
-        "ClusterConfiguration": NotRequired["ClusterConfigurationTypeDef"],
+        "Name": str,
+        "Status": str,
+        "Source": str,
+        "KmsKeyId": str,
+        "ARN": str,
+        "ClusterConfiguration": "ClusterConfigurationTypeDef",
     },
+    total=False,
 )
 
 SubnetGroupTypeDef = TypedDict(
     "SubnetGroupTypeDef",
     {
-        "Name": NotRequired[str],
-        "Description": NotRequired[str],
-        "VpcId": NotRequired[str],
-        "Subnets": NotRequired[List["SubnetTypeDef"]],
-        "ARN": NotRequired[str],
+        "Name": str,
+        "Description": str,
+        "VpcId": str,
+        "Subnets": List["SubnetTypeDef"],
+        "ARN": str,
     },
+    total=False,
 )
 
 SubnetTypeDef = TypedDict(
     "SubnetTypeDef",
     {
-        "Identifier": NotRequired[str],
-        "AvailabilityZone": NotRequired["AvailabilityZoneTypeDef"],
+        "Identifier": str,
+        "AvailabilityZone": "AvailabilityZoneTypeDef",
     },
+    total=False,
 )
 
 TagResourceRequestRequestTypeDef = TypedDict(
@@ -987,18 +1173,20 @@ TagResourceResponseTypeDef = TypedDict(
 TagTypeDef = TypedDict(
     "TagTypeDef",
     {
-        "Key": NotRequired[str],
-        "Value": NotRequired[str],
+        "Key": str,
+        "Value": str,
     },
+    total=False,
 )
 
 UnprocessedClusterTypeDef = TypedDict(
     "UnprocessedClusterTypeDef",
     {
-        "ClusterName": NotRequired[str],
-        "ErrorType": NotRequired[str],
-        "ErrorMessage": NotRequired[str],
+        "ClusterName": str,
+        "ErrorType": str,
+        "ErrorMessage": str,
     },
+    total=False,
 )
 
 UntagResourceRequestRequestTypeDef = TypedDict(
@@ -1017,14 +1205,27 @@ UntagResourceResponseTypeDef = TypedDict(
     },
 )
 
-UpdateACLRequestRequestTypeDef = TypedDict(
-    "UpdateACLRequestRequestTypeDef",
+_RequiredUpdateACLRequestRequestTypeDef = TypedDict(
+    "_RequiredUpdateACLRequestRequestTypeDef",
     {
         "ACLName": str,
-        "UserNamesToAdd": NotRequired[Sequence[str]],
-        "UserNamesToRemove": NotRequired[Sequence[str]],
     },
 )
+_OptionalUpdateACLRequestRequestTypeDef = TypedDict(
+    "_OptionalUpdateACLRequestRequestTypeDef",
+    {
+        "UserNamesToAdd": Sequence[str],
+        "UserNamesToRemove": Sequence[str],
+    },
+    total=False,
+)
+
+
+class UpdateACLRequestRequestTypeDef(
+    _RequiredUpdateACLRequestRequestTypeDef, _OptionalUpdateACLRequestRequestTypeDef
+):
+    pass
+
 
 UpdateACLResponseTypeDef = TypedDict(
     "UpdateACLResponseTypeDef",
@@ -1034,25 +1235,38 @@ UpdateACLResponseTypeDef = TypedDict(
     },
 )
 
-UpdateClusterRequestRequestTypeDef = TypedDict(
-    "UpdateClusterRequestRequestTypeDef",
+_RequiredUpdateClusterRequestRequestTypeDef = TypedDict(
+    "_RequiredUpdateClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
-        "Description": NotRequired[str],
-        "SecurityGroupIds": NotRequired[Sequence[str]],
-        "MaintenanceWindow": NotRequired[str],
-        "SnsTopicArn": NotRequired[str],
-        "SnsTopicStatus": NotRequired[str],
-        "ParameterGroupName": NotRequired[str],
-        "SnapshotWindow": NotRequired[str],
-        "SnapshotRetentionLimit": NotRequired[int],
-        "NodeType": NotRequired[str],
-        "EngineVersion": NotRequired[str],
-        "ReplicaConfiguration": NotRequired["ReplicaConfigurationRequestTypeDef"],
-        "ShardConfiguration": NotRequired["ShardConfigurationRequestTypeDef"],
-        "ACLName": NotRequired[str],
     },
 )
+_OptionalUpdateClusterRequestRequestTypeDef = TypedDict(
+    "_OptionalUpdateClusterRequestRequestTypeDef",
+    {
+        "Description": str,
+        "SecurityGroupIds": Sequence[str],
+        "MaintenanceWindow": str,
+        "SnsTopicArn": str,
+        "SnsTopicStatus": str,
+        "ParameterGroupName": str,
+        "SnapshotWindow": str,
+        "SnapshotRetentionLimit": int,
+        "NodeType": str,
+        "EngineVersion": str,
+        "ReplicaConfiguration": "ReplicaConfigurationRequestTypeDef",
+        "ShardConfiguration": "ShardConfigurationRequestTypeDef",
+        "ACLName": str,
+    },
+    total=False,
+)
+
+
+class UpdateClusterRequestRequestTypeDef(
+    _RequiredUpdateClusterRequestRequestTypeDef, _OptionalUpdateClusterRequestRequestTypeDef
+):
+    pass
+
 
 UpdateClusterResponseTypeDef = TypedDict(
     "UpdateClusterResponseTypeDef",
@@ -1078,14 +1292,27 @@ UpdateParameterGroupResponseTypeDef = TypedDict(
     },
 )
 
-UpdateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "UpdateSubnetGroupRequestRequestTypeDef",
+_RequiredUpdateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "_RequiredUpdateSubnetGroupRequestRequestTypeDef",
     {
         "SubnetGroupName": str,
-        "Description": NotRequired[str],
-        "SubnetIds": NotRequired[Sequence[str]],
     },
 )
+_OptionalUpdateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "_OptionalUpdateSubnetGroupRequestRequestTypeDef",
+    {
+        "Description": str,
+        "SubnetIds": Sequence[str],
+    },
+    total=False,
+)
+
+
+class UpdateSubnetGroupRequestRequestTypeDef(
+    _RequiredUpdateSubnetGroupRequestRequestTypeDef, _OptionalUpdateSubnetGroupRequestRequestTypeDef
+):
+    pass
+
 
 UpdateSubnetGroupResponseTypeDef = TypedDict(
     "UpdateSubnetGroupResponseTypeDef",
@@ -1095,14 +1322,27 @@ UpdateSubnetGroupResponseTypeDef = TypedDict(
     },
 )
 
-UpdateUserRequestRequestTypeDef = TypedDict(
-    "UpdateUserRequestRequestTypeDef",
+_RequiredUpdateUserRequestRequestTypeDef = TypedDict(
+    "_RequiredUpdateUserRequestRequestTypeDef",
     {
         "UserName": str,
-        "AuthenticationMode": NotRequired["AuthenticationModeTypeDef"],
-        "AccessString": NotRequired[str],
     },
 )
+_OptionalUpdateUserRequestRequestTypeDef = TypedDict(
+    "_OptionalUpdateUserRequestRequestTypeDef",
+    {
+        "AuthenticationMode": "AuthenticationModeTypeDef",
+        "AccessString": str,
+    },
+    total=False,
+)
+
+
+class UpdateUserRequestRequestTypeDef(
+    _RequiredUpdateUserRequestRequestTypeDef, _OptionalUpdateUserRequestRequestTypeDef
+):
+    pass
+
 
 UpdateUserResponseTypeDef = TypedDict(
     "UpdateUserResponseTypeDef",
@@ -1115,12 +1355,13 @@ UpdateUserResponseTypeDef = TypedDict(
 UserTypeDef = TypedDict(
     "UserTypeDef",
     {
-        "Name": NotRequired[str],
-        "Status": NotRequired[str],
-        "AccessString": NotRequired[str],
-        "ACLNames": NotRequired[List[str]],
-        "MinimumEngineVersion": NotRequired[str],
-        "Authentication": NotRequired["AuthenticationTypeDef"],
-        "ARN": NotRequired[str],
+        "Name": str,
+        "Status": str,
+        "AccessString": str,
+        "ACLNames": List[str],
+        "MinimumEngineVersion": str,
+        "Authentication": "AuthenticationTypeDef",
+        "ARN": str,
     },
+    total=False,
 )
